@@ -12,7 +12,7 @@
 ##############################################################################
 """Access control support"""
 
-__version__='$Revision: 1.55.6.1 $'[11:-2]
+__version__='$Revision: 1.56.14.4 $'[11:-2]
 
 
 from Globals import DTMLFile, MessageDialog, Dictionary
@@ -33,7 +33,7 @@ def _isNotBeingUsedAsAMethod(self):
     return not aq_get(self, '_isBeingUsedAsAMethod_', 0)
 
 class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
-    """An obect that has configurable permissions"""
+    """An object that has configurable permissions"""
 
     __ac_permissions__=(
         ('Change permissions',
@@ -90,14 +90,22 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
 
         return tuple(r)
 
-    def permission_settings(self):
-        """Return user-role permission settings
+    def permission_settings(self, permission=None):
+        """Return user-role permission settings. If 'permission' 
+           is passed to the method then only the settings for 'permission'
+           is returned.
         """
         result=[]
         valid=self.valid_roles()
         indexes=range(len(valid))
         ip=0
-        for p in self.ac_inherited_permissions(1):
+
+        permissions = self.ac_inherited_permissions(1)
+        # Filter permissions
+        if permission:
+            permissions = [p for p in permissions if p[0] == permission]
+
+        for p in permissions:
             name, value = p[:2]
             p=Permission(name,value,self)
             roles=p.getRoles(default=[])
@@ -171,7 +179,7 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
                 if REQUEST is not None: return self.manage_access(REQUEST)
                 return
 
-        raise 'Invalid Permission', (
+        raise ValueError, (
             "The permission <em>%s</em> is invalid." %
                 escape(permission_to_manage))
 
@@ -210,7 +218,7 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
         if fails:
             return MessageDialog(title="Warning!",
                                  message="Some permissions had errors: "
-                                   + ', '.join(fails),
+                                   + escape(', '.join(fails)),
                                  action='manage_access')
         return MessageDialog(
             title  ='Success!',
@@ -245,7 +253,7 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
                      },
                     valid_roles)
 
-        raise 'Invalid Permission', (
+        raise ValueError, (
             "The permission <em>%s</em> is invalid." % escape(permission))
 
     def acquiredRolesAreUsedBy(self, permission):
@@ -257,7 +265,7 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
                 roles=p.getRoles()
                 return type(roles) is ListType and 'CHECKED' or ''
 
-        raise 'Invalid Permission', (
+        raise ValueError, (
             "The permission <em>%s</em> is invalid." % escape(permission))
 
 
@@ -462,7 +470,7 @@ class RoleManager(ExtensionClass.Base, PermissionMapping.RoleManager):
             return self.manage_access(REQUEST)
 
 
-    def _delRoles(self, roles, REQUEST):
+    def _delRoles(self, roles, REQUEST=None):
         if not roles:
             return MessageDialog(
                    title  ='Incomplete',
