@@ -34,19 +34,20 @@
 #   on restart if there is still a product directory.
 
 
-import Globals, OFS.Folder, OFS.SimpleItem, os,  Acquisition, Products
-import re, zlib, Globals, cPickle, marshal, rotor
-import ZClasses, ZClasses.ZClass, AccessControl.Owned
-from urllib import quote
+import os, re, zlib, marshal, cPickle
 from cgi import escape
+from urllib import quote
 
+import Globals, OFS.Folder, OFS.SimpleItem,  Acquisition, Products
+import ZClasses, AccessControl.Owned
 from OFS.Folder import Folder
+from HelpSys.HelpSys import ProductHelp
+from AccessControl import Unauthorized
+
 from Factory import Factory
 from Permission import PermissionManager
-import ZClasses, ZClasses.ZClass
-from HelpSys.HelpSys import ProductHelp
 import RefreshFuncs
-from AccessControl import Unauthorized
+from App.config import getConfiguration
 
 
 class ProductFolder(Folder):
@@ -76,6 +77,9 @@ class ProductFolder(Folder):
     def _canCopy(self, op=0):
         return 0
 
+Globals.InitializeClass(ProductFolder)
+
+
 class Product(Folder, PermissionManager):
     """Model a product that can be created through the web.
     """
@@ -193,6 +197,7 @@ class Product(Folder, PermissionManager):
 
         id=self.id
 
+        import rotor
         import tar
         rot=rotor.newrotor(id+' shshsh')
         ar=tar.tgzarchive("%s-%s" % (id, self.version))
@@ -207,7 +212,7 @@ class Product(Folder, PermissionManager):
         # Extensions
         pp=id+'.'
         lpp=len(pp)
-        ed=os.path.join(INSTANCE_HOME,'Extensions')
+        ed=os.path.join(getConfiguration().instancehome,'Extensions')
         if os.path.exists(ed):
             for name in os.listdir(ed):
                 suffix=''
@@ -255,14 +260,15 @@ class Product(Folder, PermissionManager):
         "Product Distributions"
 
         def __bobo_traverse__(self, REQUEST, name):
-            if name[-7:] != '.tar.gz': raise 'Invalid Name', escape(name)
+            if name[-7:] != '.tar.gz':
+                raise ValueError, 'Invalid Name: %s' % escape(name)
             l=name.find('-')
             id, version = name[:l], name[l+1:-7]
             product=self.aq_parent
             if product.id==id and product.version==version:
                 return Distribution(product)
 
-            raise 'Invalid version or product id', escape(name)
+            raise ValueError, 'Invalid version or product id: %s' % escape(name)
 
     Distributions=Distributions()
 
@@ -411,6 +417,7 @@ class Product(Folder, PermissionManager):
         if REQUEST is not None:
             return self.manage_refresh(REQUEST)
 
+Globals.InitializeClass(Product)
 
 
 class CompressedOutputFile:
@@ -432,7 +439,9 @@ class CompressedInputFile:
     def __init__(self, f, rot):
         self._c=zlib.decompressobj()
         self._b=''
-        if type(rot) is type(''): rot=rotor.newrotor(rot)
+        if isinstance(rot, str):
+            import rotor
+            rot=rotor.newrotor(rot)
         self._rot=rot
         rot.decrypt('')
         self._f=f
@@ -484,12 +493,22 @@ class Distribution:
 def initializeProduct(productp, name, home, app):
     # Initialize a levered product
     products=app.Control_Panel.Products
+    fver = ''
 
     if hasattr(productp, '__import_error__'): ie=productp.__import_error__
     else: ie=None
 
-    try: fver=open(home+'/version.txt').read().strip()
-    except: fver=''
+    # Retrieve version number from any suitable version.txt
+    for fname in ('version.txt', 'VERSION.txt', 'VERSION.TXT'):
+        try:
+            fpath = os.path.join(home, fname)
+            fhandle = open(fpath, 'r')
+            fver = fhandle.read().strip()
+            fhandle.close()
+            break
+        except IOError:
+            continue
+
     old=None
     try:
         if ihasattr(products,name):
@@ -573,7 +592,5 @@ def ihasattr(o, name):
 
 
 def doInstall():
-    if os.environ.has_key('FORCE_PRODUCT_LOAD'):
-        return not not os.environ['FORCE_PRODUCT_LOAD']
+    return getConfiguration().enable_product_installation
 
-    return not os.environ.get('ZEO_CLIENT')
