@@ -12,15 +12,65 @@
 ##############################################################################
 """Encapsulation of date/time values"""
 
-__version__='$Revision: 1.79.6.6 $'[11:-2]
+__version__='$Revision: 1.85.12.10 $'[11:-2]
 
 
-import re,sys, os, math,  DateTimeZone
-from time import time, gmtime, localtime, asctime
+import re, math,  DateTimeZone
+from time import time, gmtime, localtime 
 from time import daylight, timezone, altzone, strftime
-from types import InstanceType,IntType,FloatType,StringType,UnicodeType
-try: from time import tzname
-except: tzname=('UNKNOWN','UNKNOWN')
+
+
+try:
+    from time import tzname
+except:
+    tzname=('UNKNOWN','UNKNOWN')
+
+class DateTimeError( Exception ):
+    pass
+
+class SyntaxError( DateTimeError ):
+    pass
+
+class DateError( DateTimeError ):
+    pass
+
+class TimeError( DateTimeError ):
+    pass
+
+from App.config import getConfiguration
+
+default_datefmt = None
+
+def getDefaultDateFormat():
+    global default_datefmt
+    if default_datefmt is None:
+        try:
+            default_datefmt = getConfiguration().datetime_format
+            return default_datefmt
+        except:
+            return 'us'
+    else:
+        return default_datefmt
+
+
+try:
+    from time import tzname
+except:
+    tzname=('UNKNOWN','UNKNOWN')
+
+class DateTimeError( Exception ):
+    pass
+
+class SyntaxError( DateTimeError ):
+    pass
+
+class DateError( DateTimeError ):
+    pass
+
+class TimeError( DateTimeError ):
+    pass
+
+
 
 # To control rounding errors, we round system time to the nearest
 # millisecond.  Then delicate calculations can rely on that the
@@ -225,7 +275,7 @@ class _cache:
            'bt':'GMT+3', 'zp4':'GMT+4', 'zp5':'GMT+5', 'zp6':'GMT+6',
            'wast':'GMT+7', 'cct':'GMT+8', 'jst':'GMT+9', 'east':'GMT+10',
            'gst':'GMT+10', 'nzt':'GMT+12', 'nzst':'GMT+12', 'idle':'GMT+12',
-           'ret':'GMT+4'
+           'ret':'GMT+4', 'ist': 'GMT+0530'
            }
 
     def __init__(self):
@@ -236,7 +286,7 @@ class _cache:
         try:   n=self._zmap[k.lower()]
         except KeyError:
             if numericTimeZoneMatch(k) == None:
-                raise 'DateTimeError','Unrecognized timezone: %s' % k
+                raise DateTimeError,'Unrecognized timezone: %s' % k
             return k
         try: return self._d[n]
         except KeyError:
@@ -260,7 +310,7 @@ def _findLocalTimeZoneName(isDST):
                 localzone = altzone
             else:
                 localzone = timezone
-            offset=(-localzone/(60*60))
+            offset=(-localzone/(60*60.0))
             majorOffset=int(offset)
             if majorOffset != 0 :
                 minorOffset=abs(int((offset % majorOffset) * 60.0))
@@ -388,20 +438,23 @@ def safegmtime(t):
     '''gmtime with a safety zone.'''
     try:
         t_int = int(t)
-        return  gmtime(t_int)
-    except (IOError, OverflowError):
-        raise 'TimeError', 'The time %f is beyond the range ' \
+        if isinstance(t_int, long):
+            raise OverflowError # Python 2.3 fix: int can return a long!
+        return gmtime(t_int)
+    except (ValueError, OverflowError):
+        raise TimeError, 'The time %f is beyond the range ' \
               'of this Python implementation.' % float(t)
 
 def safelocaltime(t):
     '''localtime with a safety zone.'''
     try:
         t_int = int(t)
-    except OverflowError:
-        raise 'TimeError', 'The time %f is beyond the range ' \
+        if isinstance(t_int, long):
+            raise OverflowError # Python 2.3 fix: int can return a long!
+        return localtime(t_int)
+    except (ValueError, OverflowError):
+        raise TimeError, 'The time %f is beyond the range ' \
               'of this Python implementation.' % float(t)
-    rval = localtime(t_int)
-    return rval
 
 def _tzoffset2rfc822zone(seconds):
     """Takes an offset, such as from _tzoffset(), and returns an rfc822 
@@ -409,6 +462,7 @@ def _tzoffset2rfc822zone(seconds):
        _tzoffset() is the negative of what time.localzone and time.altzone is."""
     return "%+03d%02d" % divmod( (seconds/60), 60) 
 
+
 class DateTime:
     """DateTime objects represent instants in time and provide
        interfaces for controlling its representation without
@@ -455,7 +509,13 @@ class DateTime:
     __roles__=None
     __allow_access_to_unprotected_subobjects__=1
 
-    def __init__(self,*args):
+    # Make class-specific exceptions available as attributes.
+    DateError = DateError
+    TimeError = TimeError
+    DateTimeError = DateTimeError
+    SyntaxError = SyntaxError
+
+    def __init__(self,*args, **kw):
         """Return a new date-time object
 
         A DateTime object always maintains its value as an absolute
@@ -469,9 +529,9 @@ class DateTime:
         DateTimes may be created with from zero to seven arguments.
 
 
-          - If the function is called with no arguments, then the
-            current date/time is returned, represented in the
-            timezone of the local machine.
+          - If the function is called with no arguments or with None, 
+            then the current date/time is returned, represented in the
+            timezone of the local machine. 
 
           - If the function is invoked with a single string argument
             which is a recognized timezone name, an object representing
@@ -600,6 +660,13 @@ class DateTime:
             effect of this is as if you had taken the value of time.time()
             at that time on a machine in the specified timezone).
 
+            New in Zope 2.7:
+            A new keyword parameter "datefmt" can be passed to the 
+            constructor. If set to "international", the constructor
+            is forced to treat ambigious dates as "days before month
+            before year". This useful if you need to parse non-US
+            dates in a reliable way
+
         In any case that a floating point number of seconds is given
         or derived, it's rounded to the nearest millisecond.
 
@@ -612,19 +679,19 @@ class DateTime:
         timezones recognized by the DateTime module. Recognition of
         timezone names is case-insensitive.""" #'
 
+        datefmt = kw.get('datefmt', getDefaultDateFormat())
         d=t=s=None
         ac=len(args)
         millisecs = None
 
-        if ac and args[0]==None: return
-        elif ac==10:
+        if ac==10:
             # Internal format called only by DateTime
-            yr,mo,dy,hr,mn,sc,tz,t,d,s=args
+            yr,mo,dy,hr,mn,sc,tz,t,d,s=args                  
         elif ac == 11:
             # Internal format that includes milliseconds.
             yr,mo,dy,hr,mn,sc,tz,t,d,s,millisecs=args
 
-        elif not args:
+        elif not args or (ac and args[0]==None):
             # Current time, to be displayed in local timezone
             t = time()
             lt = safelocaltime(t)
@@ -638,9 +705,9 @@ class DateTime:
             arg=args[0]
 
             if arg=='':
-                raise self.SyntaxError, arg
+                raise SyntaxError, arg
 
-            if type(arg) in [StringType,UnicodeType] and arg.lower() in self._tzinfo._zidx:
+            if isinstance(arg, (unicode, str)) and arg.lower() in self._tzinfo._zidx:
                 # Current time, to be displayed in specified timezone
                 t,tz=time(),self._tzinfo._zmap[arg.lower()]
                 ms=(t-math.floor(t))
@@ -649,19 +716,18 @@ class DateTime:
                 x = _calcDependentSecond(tz, t)
                 yr,mo,dy,hr,mn,sc = _calcYMDHMS(x, ms)
 
-            elif type(arg) in [StringType,UnicodeType]:
+            elif isinstance(arg, (unicode, str)):
                 # Date/time string
 
                 if arg.find(' ')==-1 and arg[4]=='-':
                     yr,mo,dy,hr,mn,sc,tz=self._parse_iso8601(arg)
                 else:
-                    yr,mo,dy,hr,mn,sc,tz=self._parse(arg)
-
+                    yr,mo,dy,hr,mn,sc,tz=self._parse(arg, datefmt)
 
                 if not self._validDate(yr,mo,dy):
-                    raise self.DateTimeError, 'Invalid date: %s' % arg
+                    raise DateError, 'Invalid date: %s' % arg
                 if not self._validTime(hr,mn,int(sc)):
-                    raise self.DateTimeError, 'Invalid time: %s' % arg
+                    raise TimeError, 'Invalid time: %s' % arg
                 ms = sc - math.floor(sc)
                 x = _calcDependentSecond2(yr,mo,dy,hr,mn,sc)
 
@@ -669,7 +735,7 @@ class DateTime:
                     try: tz=self._tzinfo._zmap[tz.lower()]
                     except KeyError:
                         if numericTimeZoneMatch(tz) is None:
-                            raise self.DateTimeError, \
+                            raise DateTimeError, \
                                   'Unknown time zone in date: %s' % arg
                 else:
                     tz = self._calcTimezoneName(x, ms)
@@ -686,7 +752,7 @@ class DateTime:
                 sc=sc+ms
 
         elif ac==2:
-            if type(args[1])==StringType:
+            if isinstance(args[1], str):
                 # Seconds from epoch (gmt) and timezone
                 t,tz=args
                 ms = (t - math.floor(t))
@@ -715,7 +781,7 @@ class DateTime:
             hr,mn,sc,tz=0,0,0,0
             yr = _correctYear(yr)
             if not self._validDate(yr,mo,dy):
-                raise self.DateTimeError, 'Invalid date: %s' % (args,)
+                raise DateError, 'Invalid date: %s' % (args,)
             args=args[3:]
             if args:
                 hr,args=args[0],args[1:]
@@ -726,9 +792,9 @@ class DateTime:
                         if args:
                             tz,args=args[0],args[1:]
                             if args:
-                                raise self.DateTimeError,'Too many arguments'
+                                raise DateTimeError,'Too many arguments'
             if not self._validTime(hr,mn,sc):
-                raise self.DateTimeError, 'Invalid time: %s' % `args`
+                raise TimeError, 'Invalid time: %s' % `args`
             leap = (yr % 4 == 0) and (yr % 100 != 0 or yr % 400 == 0)
 
             x = _calcDependentSecond2(yr,mo,dy,hr,mn,sc)
@@ -737,7 +803,7 @@ class DateTime:
                 try: tz=self._tzinfo._zmap[tz.lower()]
                 except KeyError:
                     if numericTimeZoneMatch(tz) is None:
-                        raise self.DateTimeError, \
+                        raise DateTimeError, \
                               'Unknown time zone: %s' % tz
             else:
                 # Get local time zone name
@@ -774,9 +840,6 @@ class DateTime:
         # self._millis is the time since the epoch
         # in long integer milliseconds.
 
-    DateTimeError='DateTimeError'
-    SyntaxError  ='Invalid Date-Time String'
-    DateError    ='Invalid Date Components'
     int_pattern  =re.compile(r'([0-9]+)') #AJ
     flt_pattern  =re.compile(r':([0-9]+\.[0-9]+)') #AJ
     name_pattern =re.compile(r'([a-zA-Z]+)', re.I) #AJ
@@ -855,11 +918,16 @@ class DateTime:
             yr = ((yr - 1970) % 28) + 1970
             x = _calcDependentSecond2(yr,mo,dy,hr,mn,sc)
             nearTime = x - fsetAtEpoch - long(EPOCH) + 86400L + ms
+
+            # nearTime might still be negative if we are east of Greenwich.
+            # But we can asume on 1969/12/31 were no timezone changes.
+            nearTime = max(0, nearTime)
+
             ltm = safelocaltime(nearTime)
         tz = self.localZone(ltm)
         return tz
 
-    def _parse(self,st):
+    def _parse(self,st, datefmt=getDefaultDateFormat()):
         # Parse date-time components from a string
         month=year=tz=tm=None
         spaces        =self.space_chars
@@ -930,21 +998,21 @@ class DateTime:
                 if MonthNumbers.has_key(s):
                     v=MonthNumbers[s]
                     if month is None: month=v
-                    else: raise self.SyntaxError, st
+                    else: raise SyntaxError, st
                     continue
                 # Check for time modifier:
                 if s in TimeModifiers:
                     if tm is None: tm=s
-                    else: raise self.SyntaxError, st
+                    else: raise SyntaxError, st
                     continue
                 # Check for and skip day of week:
                 if DayOfWeekNames.has_key(s):
                     continue
 
-            raise self.SyntaxError, st
+            raise SyntaxError, st
 
         day=None
-        if ints[-1] > 60 and d not in ['.',':'] and len(ints) > 2:
+        if ints[-1] > 60 and d not in ['.',':','/'] and len(ints) > 2:
             year=ints[-1]
             del ints[-1]
             if month:
@@ -986,8 +1054,13 @@ class DateTime:
                     day=ints[0]
                     month=ints[1]
                 else:
-                    day=ints[1]
-                    month=ints[0]
+                    if datefmt=="us":
+                        day=ints[1]
+                        month=ints[0]
+                    else:
+                        day=ints[0]
+                        month=ints[1]
+
             elif ints[0] <= 12:
                 month=ints[0]
                 day=ints[1]
@@ -999,34 +1072,34 @@ class DateTime:
             year,month,day = localtime(time())[:3]
 
         year = _correctYear(year)
-        if year < 1000: raise self.SyntaxError, st
+        if year < 1000: raise SyntaxError, st
 
         leap = year%4==0 and (year%100!=0 or year%400==0)
         try:
             if not day or day > self._month_len[leap][month]:
-                raise self.DateError, st
+                raise DateError, st
         except IndexError:
-            raise self.DateError, st
+            raise DateError, st
         tod=0
         if ints:
             i=ints[0]
             # Modify hour to reflect am/pm
             if tm and (tm=='pm') and i<12:  i=i+12
             if tm and (tm=='am') and i==12: i=0
-            if i > 24: raise self.DateTimeError, st
+            if i > 24: raise TimeError, st
             tod = tod + int(i) * 3600
             del ints[0]
             if ints:
                 i=ints[0]
-                if i > 60: raise self.DateTimeError, st
+                if i > 60: raise TimeError, st
                 tod = tod + int(i) * 60
                 del ints[0]
                 if ints:
                     i=ints[0]
-                    if i > 60: raise self.DateTimeError, st
+                    if i > 60: raise TimeError, st
                     tod = tod + i
                     del ints[0]
-                    if ints: raise self.SyntaxError,st
+                    if ints: raise SyntaxError,st
 
 
         tod_int = int(math.floor(tod))
@@ -1448,12 +1521,11 @@ class DateTime:
     def rfc822(self):
         """Return the date in RFC 822 format"""
         tzoffset = _tzoffset2rfc822zone(_tzoffset(self._tz, self._t))
-            
+
         return '%s, %2.2d %s %d %2.2d:%2.2d:%2.2d %s' % (
             self._aday,self._day,self._amon,self._year,
             self._hour,self._minute,self._nearsec,tzoffset)
 
-
     # New formats
     def fCommon(self):
         """Return a string representing the object\'s value
@@ -1526,7 +1598,7 @@ class DateTime:
         """A DateTime may be added to a number and a number may be
            added to a DateTime;  two DateTimes cannot be added."""
         if hasattr(other,'_t'):
-            raise self.DateTimeError,'Cannot add two DateTimes'
+            raise DateTimeError,'Cannot add two DateTimes'
         o=float(other)
         tz = self._tz
         t = (self._t + (o*86400.0))
@@ -1613,8 +1685,8 @@ class DateTime:
         try:
             return self.__parse_iso8601(s)
         except IndexError:
-            raise self.DateError,'Not an ISO 8601 compliant date string: "%s"' %  s
-
+            raise SyntaxError, (
+                'Not an ISO 8601 compliant date string: "%s"' % s)
 
     def __parse_iso8601(self,s):
         """ parse an ISO 8601 compliant date """
@@ -1623,30 +1695,43 @@ class DateTime:
         hour=minute=seconds=hour_off=min_off=0
 
         datereg = re.compile('([0-9]{4})(-([0-9][0-9]))?(-([0-9][0-9]))?')
-        timereg = re.compile('([0-9]{2})(:([0-9][0-9]))?(:([0-9][0-9]))?(\.[0-9]{1,20})?')
+        timereg = re.compile('T([0-9]{2})(:([0-9][0-9]))?(:([0-9][0-9]))?(\.[0-9]{1,20})?')
+        zonereg = re.compile('([+-][0-9][0-9])(:?([0-9][0-9]))')
 
         # Date part
 
-        fields = datereg.split(s.strip())
+        fields = datereg.split(s.strip(), 1)
 
         if fields[1]:   year  = int(fields[1])
         if fields[3]:   month = int(fields[3])
         if fields[5]:   day   = int(fields[5])
+        t = fields[6]
+        if t:
+            if not fields[5]:
+                # Specifying time requires specifying a day.
+                raise IndexError
 
-        if s.find('T')>-1:
-            fields = timereg.split(s[s.find('T')+1:])
+            fields = timereg.split(t)
 
             if fields[1]:   hour     = int(fields[1])
             if fields[3]:   minute   = int(fields[3])
             if fields[5]:   seconds  = int(fields[5])
             if fields[6]:   seconds  = seconds+float(fields[6])
+            z = fields[7]
 
-        if s.find('Z')>-1:
-            pass
+            if z and z.startswith('Z'):
+                # Waaaa! This is wrong, since 'Z' and '+HH:MM'
+                # are supposed to be mutually exclusive.
+                # It's only here to prevent breaking 2.7 beta.
+                z = z[1:]
 
-        if s[-3]==':' and s[-6] in ['+','-']:
-            hour_off = int(s[-6:-3])
-            min_off  = int(s[-2:])
+            if z:
+                fields = zonereg.split(z)
+                hour_off = int(fields[1])
+                min_off  = int(fields[3])
+                if fields[4]:
+                    # Garbage after time zone
+                    raise IndexError
 
         return year,month,day,hour,minute,seconds,'GMT%+03d%02d' % (hour_off,min_off)
 
@@ -1685,3 +1770,4 @@ class strftimeFormatter:
 def Timezones():
     """Return the list of recognized timezone names"""
     return _cache._zlst
+
