@@ -11,17 +11,19 @@
 #
 ##############################################################################
 __doc__="""Copy interface"""
-__version__='$Revision: 1.82.6.1 $'[11:-2]
+__version__='$Revision: 1.85.2.6 $'[11:-2]
 
 import sys,  Globals, Moniker, tempfile, ExtensionClass
 from marshal import loads, dumps
 from urllib import quote, unquote
 from zlib import compress, decompress
+
 from App.Dialogs import MessageDialog
 from AccessControl import getSecurityManager
 from Acquisition import aq_base, aq_inner, aq_parent
-from zExceptions import Unauthorized
-from AccessControl import getSecurityManager
+from zExceptions import Unauthorized, BadRequest
+from webdav.Lockable import ResourceLockedError
+from cgi import escape
 
 CopyError='Copy Error'
 
@@ -31,8 +33,10 @@ class CopyContainer(ExtensionClass.Base):
 
     __ac_permissions__=(
         ('View management screens',
-         ('manage_cutObjects', 'manage_copyObjects', 'manage_pasteObjects',
+         ('manage_copyObjects', 'manage_pasteObjects',
           'manage_renameForm', 'manage_renameObject', 'manage_renameObjects',)),
+        ('Delete objects',
+         ('manage_cutObjects',)),
         )
 
 
@@ -65,8 +69,12 @@ class CopyContainer(ExtensionClass.Base):
         oblist=[]
         for id in ids:
             ob=self._getOb(id)
+
+            if ob.wl_isLocked():
+                raise ResourceLockedError, 'Object "%s" is locked via WebDAV' % ob.getId()
+
             if not ob.cb_isMoveable():
-                raise CopyError, eNotSupported % id
+                raise CopyError, eNotSupported % escape(id)
             m=Moniker.Moniker(ob)
             oblist.append(m.dump())
         cp=(1, oblist)
@@ -91,7 +99,7 @@ class CopyContainer(ExtensionClass.Base):
         for id in ids:
             ob=self._getOb(id)
             if not ob.cb_isCopyable():
-                raise CopyError, eNotSupported % id
+                raise CopyError, eNotSupported % escape(id)
             m=Moniker.Moniker(ob)
             oblist.append(m.dump())
         cp=(0, oblist)
@@ -150,7 +158,7 @@ class CopyContainer(ExtensionClass.Base):
             # Copy operation
             for ob in oblist:
                 if not ob.cb_isCopyable():
-                    raise CopyError, eNotSupported % ob.getId()
+                    raise CopyError, eNotSupported % escape(ob.getId())
                 try:    ob._notifyOfCopyTo(self, op=0)
                 except: raise CopyError, MessageDialog(
                     title='Copy Error',
@@ -164,6 +172,7 @@ class CopyContainer(ExtensionClass.Base):
                 self._setObject(id, ob)
                 ob = self._getOb(id)
                 ob.manage_afterClone(ob)
+                ob.wl_clearLocks()
 
             if REQUEST is not None:
                 return self.manage_main(self, REQUEST, update_menu=1,
@@ -174,7 +183,7 @@ class CopyContainer(ExtensionClass.Base):
             for ob in oblist:
                 id=ob.getId()
                 if not ob.cb_isMoveable():
-                    raise CopyError, eNotSupported % id
+                    raise CopyError, eNotSupported % escape(id)
                 try:    ob._notifyOfCopyTo(self, op=1)
                 except: raise CopyError, MessageDialog(
                     title='Move Error',
@@ -215,7 +224,7 @@ class CopyContainer(ExtensionClass.Base):
     def manage_renameObjects(self, ids=[], new_ids=[], REQUEST=None):
         """Rename several sub-objects"""
         if len(ids) != len(new_ids):
-            raise 'Bad Request','Please rename each listed object.'
+            raise BadRequest,'Please rename each listed object.'
         for i in range(len(ids)):
             if ids[i] != new_ids[i]:
                 self.manage_renameObject(ids[i], new_ids[i], REQUEST)
@@ -231,8 +240,10 @@ class CopyContainer(ExtensionClass.Base):
                       message=sys.exc_info()[1],
                       action ='manage_main')
         ob=self._getOb(id)
+        if ob.wl_isLocked():
+            raise ResourceLockedError, 'Object "%s" is locked via WebDAV' % ob.getId()
         if not ob.cb_isMoveable():
-            raise CopyError, eNotSupported % id
+            raise CopyError, eNotSupported % escape(id)
         self._verifyObjectPaste(ob)
         try:    ob._notifyOfCopyTo(self, op=1)
         except: raise CopyError, MessageDialog(
@@ -259,7 +270,7 @@ class CopyContainer(ExtensionClass.Base):
     def manage_clone(self, ob, id, REQUEST=None):
         # Clone an object, creating a new object with the given id.
         if not ob.cb_isCopyable():
-            raise CopyError, eNotSupported % ob.getId()
+            raise CopyError, eNotSupported % escape(ob.getId())
         try: self._checkId(id)
         except: raise CopyError, MessageDialog(
                       title='Invalid Id',
@@ -312,71 +323,74 @@ class CopyContainer(ExtensionClass.Base):
         # existing context, such as checking an object during an import
         # (the object will not yet have been connected to the acquisition
         # heirarchy).
+
         if not hasattr(object, 'meta_type'):
             raise CopyError, MessageDialog(
-                  title='Not Supported',
-                  message='The object <EM>%s</EM> does not support this' \
-                          ' operation' % absattr(object.id),
-                  action='manage_main')
-        mt=object.meta_type
+                  title   = 'Not Supported',
+                  message = ('The object <EM>%s</EM> does not support this' \
+                             ' operation' % escape(absattr(object.id))),
+                  action  = 'manage_main')
+
         if not hasattr(self, 'all_meta_types'):
             raise CopyError, MessageDialog(
-                  title='Not Supported',
-                  message='Cannot paste into this object.',
-                  action='manage_main')
+                  title   = 'Not Supported',
+                  message = 'Cannot paste into this object.',
+                  action  = 'manage_main')
+
+        method_name = None
+        mt_permission = None
+        meta_types = absattr(self.all_meta_types)
 
-        method_name=None
-        mt_permission=None
-        meta_types=absattr(self.all_meta_types)
         for d in meta_types:
-            if d['name']==mt:
-                method_name=d['action']
-                mt_permission=d.get( 'permission', None )
+            if d['name'] == object.meta_type:
+                method_name = d['action']
+                mt_permission = d.get('permission')
                 break
 
-        if mt_permission is not None:
-            if getSecurityManager().checkPermission( mt_permission, self ):
-                if not validate_src:
-                    return
-                # Ensure the user is allowed to access the object on the
-                # clipboard.
-                try:    parent=aq_parent(aq_inner(object))
-                except: parent=None
-                if getSecurityManager().validate(None, parent, None, object):
-                    return
-                raise Unauthorized, absattr(object.id)
-            else:
-                raise Unauthorized(permission=mt_permission)
-        #
-        #   XXX:    Ancient cruft, left here in true co-dependent fashion
-        #           to keep from breaking old products which don't put
-        #           permissions on their metadata registry entries.
-        #
-        if method_name is not None:
-            meth=self.unrestrictedTraverse(method_name)
-            if hasattr(meth, 'im_self'):
-                parent = meth.im_self
-            else:
-                try:    parent=aq_parent(aq_inner(meth))
-                except: parent=None
-            if getSecurityManager().validate(None, parent, None, meth):
-                # Ensure the user is allowed to access the object on the
-                # clipboard.
-                if not validate_src:
-                    return
-                try:    parent=aq_parent(aq_inner(object))
-                except: parent=None
-                if getSecurityManager().validate(None, parent, None, object):
-                    return
-                raise Unauthorized, absattr(object.id)
-            else:
-                raise Unauthorized, method_name
+        if method_name:
+            try:
+                method = self.restrictedTraverse(method_name)
+                # method_name is e.g.
+                # "manage_addProduct/PageTemplates/manage_addPageTemplateForm".
+                # restrictedTraverse will raise Unauthorized if it
+                # can't obtain the factory method by name due to a
+                # security restriction.  We depend on this side effect
+                # here!  Note that we use restrictedTraverse as
+                # opposed to checkPermission to take into account the
+                # special security circumstances related to proxy
+                # roles.  See collector #78.
 
-        raise CopyError, MessageDialog(
-              title='Not Supported',
-              message='The object <EM>%s</EM> does not support this ' \
-                      'operation.' % absattr(object.id),
-              action='manage_main')
+            except Unauthorized:
+                if mt_permission:
+                    message = ('You do not possess the %s permission in the '
+                               'context of the container into which you are '
+                               'pasting, thus you are not able to perform '
+                               'this operation.' % mt_permission)
+                else:
+                    message = ('You do not possess the permission required '
+                               'to call %s in the context of the container '
+                               'into which you are pasting, thus you are not '
+                               'able to perform this operation.' % method_name)
+
+                raise CopyError, MessageDialog(
+                  title = 'Insufficient Privileges',
+                  message = message,
+                  action = 'manage_main')
+
+            if validate_src:
+                # Ensure the user is allowed to access the object on the
+                # clipboard.
+                try:    parent = aq_parent(aq_inner(object))
+                except: parent = None
+                if not getSecurityManager().validate(None,parent,None,object):
+                    raise Unauthorized, absattr(object.id)
+
+        else: # /if method_name
+            raise CopyError, MessageDialog(
+                  title   = 'Not Supported',
+                  message = ('The object <EM>%s</EM> does not support this '
+                             'operation.' % escape(absattr(object.id))),
+                  action  = 'manage_main')
 
 Globals.default__class_init__(CopyContainer)
 
@@ -497,11 +511,11 @@ def cookie_path(request):
 fMessageDialog=Globals.HTML("""
 <HTML>
 <HEAD>
-<TITLE><dtml-var title></TITLE>
+<TITLE>&dtml-title;</TITLE>
 </HEAD>
 <BODY BGCOLOR="#FFFFFF">
-<FORM ACTION="<dtml-var action>" METHOD="GET" <dtml-if
- target>TARGET="<dtml-var target>"</dtml-if>>
+<FORM ACTION="&dtml-action;" METHOD="GET" <dtml-if
+ target>TARGET="&dtml-target;"</dtml-if>>
 <TABLE BORDER="0" WIDTH="100%%" CELLPADDING="10">
 <TR>
   <TD VALIGN="TOP">
