@@ -13,9 +13,9 @@
 
 """Property sheets"""
 
-__version__='$Revision: 1.87.6.1 $'[11:-2]
+__version__='$Revision: 1.89.4.6 $'[11:-2]
 
-import time,  App.Management, Globals
+import time,  App.Management, Globals, sys
 from webdav.WriteLockInterface import WriteLockInterface
 from ZPublisher.Converters import type_converters
 from Globals import DTMLFile, MessageDialog
@@ -28,7 +28,13 @@ from Traversable import Traversable
 from Acquisition import aq_base
 from AccessControl import getSecurityManager
 from webdav.common import isDavCollection
+from zExceptions import BadRequest, Redirect
 from cgi import escape
+from types import ListType
+
+# DM: we would like to import this from somewhere
+BadRequestException= 'Bad Request'
+
 
 class View(App.Management.Tabs, Base):
     """A view of an object, typically used for management purposes
@@ -185,16 +191,16 @@ class PropertySheet(Traversable, Persistent, Implicit):
         # systems.
         self._wrapperCheck(value)
         if not self.valid_property_id(id):
-            raise 'Bad Request', 'Invalid property id, %s.' % escape(id)
+            raise BadRequest, 'Invalid property id, %s.' % escape(id)
 
         if not self.property_extensible_schema__():
-            raise 'Bad Request', (
+            raise BadRequest, (
                 'Properties cannot be added to this property sheet')
         pself=self.p_self()
         self=self.v_self()
         if hasattr(aq_base(self),id):
             if not (id=='title' and not self.__dict__.has_key(id)):
-                raise 'Bad Request', (
+                raise BadRequest, (
                     'Invalid property id, <em>%s</em>. It is in use.' %
                         escape(id))
         if meta is None: meta={}
@@ -202,12 +208,16 @@ class PropertySheet(Traversable, Persistent, Implicit):
         pself._properties=pself._properties+(prop,)
         if type in ('selection', 'multiple selection'):
             if not value:
-                raise 'Bad Request', (
+                raise BadRequest, (
                     'The value given for a new selection property '
                     'must be a variable name<p>')
             prop['select_variable']=value
             if type=='selection': value=None
             else: value=[]
+
+        # bleah - can't change kw name in api, so use ugly workaround.
+        if sys.modules['__builtin__'].type(value) == ListType:
+            value = tuple(value)
         setattr(self, id, value)
 
     def _updateProperty(self, id, value, meta=None):
@@ -217,10 +227,10 @@ class PropertySheet(Traversable, Persistent, Implicit):
         # it will used to _replace_ the properties meta data.
         self._wrapperCheck(value)
         if not self.hasProperty(id):
-            raise 'Bad Request', 'The property %s does not exist.' % escape(id)
+            raise BadRequest, 'The property %s does not exist.' % escape(id)
         propinfo=self.propertyInfo(id)
         if not 'w' in propinfo.get('mode', 'wd'):
-            raise 'Bad Request', '%s cannot be changed.' % escape(id)
+            raise BadRequest, '%s cannot be changed.' % escape(id)
         if type(value)==type(''):
             proptype=propinfo.get('type', 'string')
             if type_converters.has_key(proptype):
@@ -232,19 +242,22 @@ class PropertySheet(Traversable, Persistent, Implicit):
                 if prop['id']==id: prop['meta']=meta
                 props.append(prop)
             pself._properties=tuple(props)
+
+        if type(value) == ListType:
+            value = tuple(value)
         setattr(self.v_self(), id, value)
 
     def _delProperty(self, id):
         # Delete the property with the given id. If a property with the
         # given id does not exist, a ValueError is raised.
         if not self.hasProperty(id):
-            raise 'Bad Request', 'The property %s does not exist.' % escape(id)
+            raise BadRequest, 'The property %s does not exist.' % escape(id)
         vself=self.v_self()
         if hasattr(vself, '_reserved_names'):
             nd=vself._reserved_names
         else: nd=()
         if (not 'd' in self.propertyInfo(id).get('mode', 'wd')) or (id in nd):
-            raise 'Bad Request', '%s cannot be deleted.' % escape(id)
+            raise BadRequest, '%s cannot be deleted.' % escape(id)
         delattr(vself, id)
         pself=self.p_self()
         pself._properties=tuple(filter(lambda i, n=id: i['id'] != n,
@@ -306,9 +319,9 @@ class PropertySheet(Traversable, Persistent, Implicit):
             value=self.getProperty(name)
 
             if type=='tokens':
-                value=' '.join(str(value))
+                value=' '.join(map(str, value))
             elif type=='lines':
-                value='\n'.join(str(value))
+                value='\n'.join(map(str, value))
             # check for xml property
             attrs=item.get('meta', {}).get('__xml_attrs__', None)
             if attrs is not None:
@@ -318,10 +331,9 @@ class PropertySheet(Traversable, Persistent, Implicit):
                 # Quote non-xml items here?
                 attrs=''
 
-            if hasattr(self,"dav__"+name):
-                prop='  <n:%s%s>%s</n:%s>' % (name, attrs, value, name)
-            else:
-                prop='  <n:%s%s>%s</n:%s>' % (name, attrs, xml_escape(value), name)
+            if not hasattr(self,"dav__"+name):
+                value = xml_escape(value)
+            prop='  <n:%s%s>%s</n:%s>' % (name, attrs, value, name)
 
             result.append(prop)
         if not result: return ''
@@ -358,9 +370,9 @@ class PropertySheet(Traversable, Persistent, Implicit):
             name, type=item['id'], item.get('type','string')
             value=self.getProperty(name)
             if type=='tokens':
-                value=' '.join(str(value))
+                value=' '.join(map(str, value))
             elif type=='lines':
-                value='\n'.join(str(value))
+                value='\n'.join(map(str, value))
             # allow for xml properties
             attrs=item.get('meta', {}).get('__xml_attrs__', None)
             if attrs is not None:
@@ -369,8 +381,10 @@ class PropertySheet(Traversable, Persistent, Implicit):
             else:
                 # quote non-xml items here?
                 attrs=''
+            if not hasattr(self, 'dav__%s' % name):
+                value = xml_escape(value)
             prop='<n:%s%s xmlns:n="%s">%s</n:%s>\n' % (
-                 name, attrs, xml_id, value, name)
+                name, attrs, xml_id, value, name)
             code='200 OK'
             if not result.has_key(code):
                 result[code]=[prop]
@@ -387,7 +401,7 @@ class PropertySheet(Traversable, Persistent, Implicit):
     manage=DTMLFile('dtml/properties', globals())
     def manage_propertiesForm(self, URL1):
         " "
-        raise 'Redirect', URL1+'/manage'
+        raise Redirect, URL1+'/manage'
 
     def manage_addProperty(self, id, value, type, REQUEST=None):
         """Add a new property via the web. Sets a new property with
@@ -424,7 +438,7 @@ class PropertySheet(Traversable, Persistent, Implicit):
         for name, value in props.items():
             if self.hasProperty(name):
                 if not 'w' in propdict[name].get('mode', 'wd'):
-                    raise 'BadRequest', '%s cannot be changed' % escape(name)
+                    raise BadRequest, '%s cannot be changed' % escape(name)
                 self._updateProperty(name, value)
         if REQUEST is not None:
             return MessageDialog(
@@ -551,10 +565,14 @@ class DAVProperties(Virtual, PropertySheet, View):
         return ''
 
     def dav__supportedlock(self):
-        return '\n  <n:lockentry>\n' \
-               '  <d:lockscope><d:exclusive/></d:lockscope>\n' \
-               '  <d:locktype><d:write/></d:locktype>\n' \
-               '  </n:lockentry>\n  '
+        vself = self.v_self()
+        out = '\n'
+        if WriteLockInterface.isImplementedBy(vself):
+            out += ('  <n:lockentry>\n'
+                    '  <d:lockscope><d:exclusive/></d:lockscope>\n'
+                    '  <d:locktype><d:write/></d:locktype>\n'
+                    '  </n:lockentry>\n  ')
+        return out
 
     def dav__lockdiscovery(self):
         security = getSecurityManager()
@@ -600,6 +618,9 @@ class PropertySheets(Traversable, Implicit, App.Management.Tabs):
         )
 
 
+    # optionally to be overridden by derived classes
+    PropertySheetClass= PropertySheet
+
     webdav =DAVProperties()
     def _get_defaults(self):
         return (self.webdav,)
@@ -639,12 +660,14 @@ class PropertySheets(Traversable, Implicit, App.Management.Tabs):
                 return propset.__of__(self)
         return default
 
-    def manage_addPropertySheet(self, id, ns):
+    def manage_addPropertySheet(self, id, ns, REQUEST=None):
         """ """
         md={'xmlns':ns}
-        ps=PropertySheet(id, md)
+        ps= self.PropertySheetClass(id, md)
         self.addPropertySheet(ps)
-        return 'OK'
+        if REQUEST is None: return ps
+        ps= self.get(id)
+        REQUEST.RESPONSE.redirect('%s/manage' % ps.absolute_url())
 
     def addPropertySheet(self, propset):
         propsets=self.aq_parent.__propsets__
@@ -656,7 +679,26 @@ class PropertySheets(Traversable, Implicit, App.Management.Tabs):
         for propset in self.aq_parent.__propsets__:
             if propset.getId() != name and  propset.xml_namespace() != name:
                 result.append(propset)
-        self.parent.__propsets__=tuple(result)
+        self.aq_parent.__propsets__=tuple(result)
+
+    ## DM: deletion support
+    def isDeletable(self,name):
+        '''currently, we say that *name* is deletable when it is not a
+        default sheet. Later, we may further restrict deletability
+        based on an instance attribute.'''
+        ps= self.get(name)
+        if ps is None: return 0
+        if ps in self._get_defaults(): return 0
+        return 1
+
+    def manage_delPropertySheets(self, ids=(), REQUEST=None):
+        '''delete all sheets identified by *ids*.'''
+        for id in ids:
+            if not self.isDeletable(id):
+                raise BadRequestException, 'attempt to delete undeletable property sheet: ' + id
+            self.delPropertySheet(id)
+        if REQUEST is not None: 
+            REQUEST.RESPONSE.redirect('%s/manage' % self.absolute_url())
 
     def __len__(self):
         return len(self.__propsets__())
@@ -677,7 +719,7 @@ class PropertySheets(Traversable, Implicit, App.Management.Tabs):
         if r is None:
             pre='../'
         else:
-            pre=r['URL']
+            pre=r['URLPATH0']
             for i in (1,2):
                 l=pre.rfind('/')
                 if l >= 0:
