@@ -12,15 +12,16 @@
 ##############################################################################
 """Rendering object hierarchies as Trees
 """
-__rcs_id__='$Id: TreeTag.py,v 1.53 2002/08/14 22:02:25 mj Exp $'
-__version__='$Revision: 1.53 $'[11:-2]
+__rcs_id__='$Id: TreeTag.py,v 1.53.68.5 2004/04/29 18:58:56 caseman Exp $'
+__version__='$Revision: 1.53.68.5 $'[11:-2]
 
 from DocumentTemplate.DT_Util import *
 from DocumentTemplate.DT_String import String
 
+from cPickle import dumps
 from string import translate
 from urllib import quote, unquote
-from zlib import compress, decompress
+from zlib import compress, decompressobj
 from binascii import b2a_base64, a2b_base64
 import re
 
@@ -92,8 +93,20 @@ String.commands['tree']=Tree
 
 pyid=id # Copy builtin
 
+simple_types = {str: 1, unicode: 1, int: 1, float: 1, long: 1,
+                tuple: 1, list: 1, dict: 1}
+
+def try_call_attr(ob, attrname, simple_types=simple_types):
+    attr = getattr(ob, attrname)
+    if type(attr) in simple_types:
+        return attr
+    try:
+        return attr()
+    except TypeError:
+        return attr
+
 def tpRender(self, md, section, args,
-             simple_type={type(''):0, type(1):0, type(1.0):0}.has_key):
+             try_call_attr=try_call_attr):
     """Render data organized as a tree.
 
     We keep track of open nodes using a cookie.  The cookie stored the
@@ -103,23 +116,22 @@ def tpRender(self, md, section, args,
       ['eagle'], # eagle is open
       ['eagle'], ['jeep', [1983, 1985]]  # eagle, jeep, 1983 jeep and 1985 jeep
 
-    where the items are object ids. The state will be converted to a
+    where the items are object ids. The state will be pickled to a
     compressed and base64ed string that gets unencoded, uncompressed,
-    and evaluated on the other side.
+    and unpickled on the other side.
 
     Note that ids used in state need not be connected to urls, since
     state manipulation is internal to rendering logic.
 
-    Note that to make eval safe, we do not allow the character '*' in
-    the state.
+    Note that to make unpickling safe, we use the MiniPickle module,
+    that only creates safe objects
     """
 
     data=[]
 
     idattr=args['id']
     if hasattr(self, idattr):
-        id=getattr(self, idattr)
-        if not simple_type(type(id)): id=id()
+        id = try_call_attr(self, idattr)
     elif hasattr(self, '_p_oid'): id=oid(self)
     else: id=pyid(self)
 
@@ -206,7 +218,7 @@ def tpRender(self, md, section, args,
 
 def tpRenderTABLE(self, id, root_url, url, state, substate, diff, data,
                   colspan, section, md, treeData, level=0, args=None,
-                  simple_type={type(''):0, type(1):0, type(1.0):0}.has_key,
+                  try_call_attr=try_call_attr,
                   ):
     "Render a tree as a table"
 
@@ -216,8 +228,7 @@ def tpRenderTABLE(self, id, root_url, url, state, substate, diff, data,
     if level >= 0:
         urlattr=args['url']
         if urlattr and hasattr(self, urlattr):
-            tpUrl=getattr(self, urlattr)
-            if not simple_type(type(tpUrl)): tpUrl=tpUrl()
+            tpUrl = try_call_attr(self, urlattr)
             url = (url and ('%s/%s' % (url, tpUrl))) or tpUrl
             root_url = root_url or tpUrl
 
@@ -323,7 +334,7 @@ def tpRenderTABLE(self, id, root_url, url, state, substate, diff, data,
 
             ####################################
             # Mostly inline encode_seq for speed
-            s=compress(str(diff))
+            s=compress(dumps(diff,1))
             if len(s) > 57: s=encode_str(s)
             else:
                 s=b2a_base64(s)[:-1]
@@ -332,7 +343,7 @@ def tpRenderTABLE(self, id, root_url, url, state, substate, diff, data,
             s=translate(s, tplus)
             ####################################
 
-            script=md['SCRIPT_NAME']
+            script=md['BASEPATH1']
 
             # Propagate extra args through tree.
             if args.has_key( 'urlparam' ):
@@ -438,8 +449,7 @@ def tpRenderTABLE(self, id, root_url, url, state, substate, diff, data,
             ids={}
             for item in items:
                 if hasattr(item, idattr):
-                    id=getattr(item, idattr)
-                    if not simple_type(type(id)): id=id()
+                    id = try_call_attr(item, idattr)
                 elif hasattr(item, '_p_oid'): id=oid(item)
                 else: id=pyid(item)
                 if len(sub)==1: sub.append([])
@@ -514,7 +524,7 @@ def apply_diff(state, diff, expand):
 
 def encode_seq(state):
     "Convert a sequence to an encoded string"
-    state=compress(str(state))
+    state=compress(dumps(state))
     l=len(state)
 
     if l > 57:
@@ -574,10 +584,22 @@ def decode_seq(state):
         state=a2b_base64(state)
 
     state=decompress(state)
-    if state.find('*') >= 0: raise 'Illegal State', state
-    try: return list(eval(state,{'__builtins__':{}}))
-    except: return []
+    try:
+        return list(MiniUnpickler(StringIO(state)).load())
+    except:
+        return []
 
+def decompress(input,max_size=10240):
+    # This sillyness can go away in python 2.2
+    d = decompressobj()
+    output = ''
+    while input:
+        fragment_size = max(1,(max_size-len(output))/1000)
+        fragment,input = input[:fragment_size],input[fragment_size:]
+        output += d.decompress(fragment)
+        if len(output)>max_size:
+            raise ValueError('Compressed input too large')
+    return output+d.flush()
 
 def tpStateLevel(state, level=0):
     for sub in state:
@@ -586,7 +608,7 @@ def tpStateLevel(state, level=0):
     return level
 
 def tpValuesIds(self, get_items, args,
-                simple_type={type(''):0, type(1):0, type(1.0):0}.has_key,
+                try_call_attr=try_call_attr,
                 ):
     # get_item(node) is a function that returns the subitems of node
 
@@ -603,8 +625,7 @@ def tpValuesIds(self, get_items, args,
                 if get_items(item):
 
                     if hasattr(item, idattr):
-                        id=getattr(item, idattr)
-                        if not simple_type(type(id)): id=id()
+                        id = try_call_attr(item, idattr)
                     elif hasattr(item, '_p_oid'): id=oid(item)
                     else: id=pyid(item)
 
@@ -624,3 +645,75 @@ def oid(self):
 #icoSpace='<IMG SRC="Blank_icon" BORDER="0">'
 #icoPlus ='<IMG SRC="Plus_icon" BORDER="0">'
 #icoMinus='<IMG SRC="Minus_icon" BORDER="0">'
+
+
+
+
+
+###############################################################################
+## Everthing below here should go in a MiniPickle.py module, but keeping it
+## internal makes an easier patch
+
+
+import pickle
+from cStringIO import StringIO
+
+
+if pickle.format_version!="2.0":
+    # Maybe the format changed, and opened a security hole
+    raise 'Invalid pickle version'
+
+
+class MiniUnpickler(pickle.Unpickler):
+    """An unpickler that can only handle simple types.
+    """
+    def refuse_to_unpickle(self):
+        raise pickle.UnpicklingError, 'Refused'
+
+    dispatch = pickle.Unpickler.dispatch.copy()
+
+    for k,v in dispatch.items():
+        if k=='' or k in '().012FGIJKLMNSTUVX]adeghjlpqrstu}':
+            # This key is necessary and safe, so leave it in the map
+            pass
+        else:
+            dispatch[k] = refuse_to_unpickle
+            # Anything unnecessary is banned, but here is some logic to explain why
+            if k in [pickle.GLOBAL, pickle.OBJ, pickle.INST, pickle.REDUCE, pickle.BUILD]:
+                # These are definite security holes
+                pass
+            elif k in [pickle.PERSID, pickle.BINPERSID]:
+                # These are just unnecessary
+                pass
+    del k
+    del v
+
+def _should_succeed(x,binary=1):
+    if x != MiniUnpickler(StringIO(dumps(x,binary))).load():
+        raise ValueError(x)
+
+def _should_fail(x,binary=1):
+    try:
+        MiniUnpickler(StringIO(dumps(x,binary))).load()
+        raise ValueError(x)
+    except pickle.UnpicklingError, e:
+        if e[0]!='Refused': raise ValueError(x)
+
+class _junk_class: pass
+
+def _test():
+    _should_succeed('hello', binary=0)
+    _should_succeed('hello')
+    _should_succeed(1)
+    _should_succeed(1L)
+    _should_succeed(1.0)
+    _should_succeed((1,2,3))
+    _should_succeed([1,2,3])
+    _should_succeed({1:2,3:4})
+    _should_fail(open)
+    _should_fail(_junk_class)
+    _should_fail(_junk_class())
+
+# Test MiniPickle on every import
+# XXX This *really* should be moved to a unittest!
+_test()
