@@ -13,48 +13,43 @@
 ##############################################################################
 """Database connection support
 
-$Id: Connection.py,v 1.72.6.3 2003/01/30 23:25:13 bwarsaw Exp $"""
+$Id: Connection.py,v 1.98.4.6 2004/05/21 16:14:04 tim_one Exp $"""
 
-from cPickleCache import PickleCache, MUCH_RING_CHECKING
-from POSException import ConflictError, ReadConflictError
+from cPickleCache import PickleCache
+from POSException import ConflictError, ReadConflictError, TransactionError
 from ExtensionClass import Base
 import ExportImport, TmpStore
 from zLOG import LOG, ERROR, BLATHER, WARNING
 from coptimizations import new_persistent_id
 from ConflictResolution import ResolvedSerial
+from Transaction import Transaction, get_transaction
+from ZODB.utils import oid_repr
 
 from cPickle import Unpickler, Pickler
 from cStringIO import StringIO
 import sys
+import threading
 from time import time
 from types import StringType, ClassType
 
 global_code_timestamp = 0
 
-if MUCH_RING_CHECKING:
-    # To get rid of this warning, change the define inside
-    # cPickleCache.c and recompile.
-    LOG('ZODB',WARNING,
-        'Using cPickleCache with low performance (but extra debugging checks)')
-del MUCH_RING_CHECKING
-
 def updateCodeTimestamp():
-    '''
-    Called after changes are made to persistence-based classes.
-    Causes all connection caches to be re-created as the
-    connections are reopened.
-    '''
+    """Called after changes are made to persistence-based classes.
+
+    Causes all connection caches to be re-created as the connections are
+    reopened.
+    """
     global global_code_timestamp
     global_code_timestamp = time()
 
-ExtensionKlass=Base.__class__
+ExtensionKlass = Base.__class__
 
-class Connection(ExportImport.ExportImport):
+class Connection(ExportImport.ExportImport, object):
     """Object managers for individual object space.
 
     An object space is a version of collection of objects.  In a
-    multi-threaded application, each thread get's it's own object
-    space.
+    multi-threaded application, each thread gets its own object space.
 
     The Connection manages movement of objects in and out of object storage.
     """
@@ -62,6 +57,7 @@ class Connection(ExportImport.ExportImport):
     _debug_info=()
     _opened=None
     _code_timestamp = 0
+    _transaction = None
 
     # Experimental. Other connections can register to be closed
     # when we close by putting something here.
@@ -79,14 +75,42 @@ class Connection(ExportImport.ExportImport):
             # XXX Why do we want version caches to behave this way?
 
             self._cache.cache_drain_resistance = 100
-        self._incrgc=self.cacheGC=cache.incrgc
-        self._invalidated=d={}
-        self._invalid=d.has_key
-        self._committed=[]
+        self._incrgc = self.cacheGC = cache.incrgc
+        self._committed = []
         self._code_timestamp = global_code_timestamp
         self._load_count = 0   # Number of objects unghosted
         self._store_count = 0  # Number of objects stored
 
+        # _invalidated queues invalidate messages delivered from the DB
+        # _inv_lock prevents one thread from modifying the set while
+        # another is processing invalidations.  All the invalidations
+        # from a single transaction should be applied atomically, so
+        # the lock must be held when reading _invalidated.
+
+        # XXX It sucks that we have to hold the lock to read
+        # _invalidated.  Normally, _invalidated is written by call
+        # dict.update, which will execute atomically by virtue of the
+        # GIL.  But some storage might generate oids where hash or
+        # compare invokes Python code.  In that case, the GIL can't
+        # save us.
+        self._inv_lock = threading.Lock()
+        self._invalidated = d = {}
+        self._invalid = d.has_key
+        self._conflicts = {}
+
+    def getTransaction(self):
+        t = self._transaction
+        if t is None:
+            # Fall back to thread-bound transactions
+            t = get_transaction()
+        return t
+
+    def setLocalTransaction(self):
+        """Use a transaction bound to the connection rather than the thread"""
+        if self._transaction is None:
+            self._transaction = Transaction()
+        return self._transaction
+
     def _cache_items(self):
         # find all items on the lru list
         items = self._cache.lru_items()
@@ -106,12 +130,14 @@ class Connection(ExportImport.ExportImport):
         return '<Connection at %08x%s>' % (id(self), ver)
 
     def _breakcr(self):
-        try: del self._cache
-        except: pass
-        try: del self._incrgc
-        except: pass
-        try: del self.cacheGC
-        except: pass
+        # Persistent objects and the cache don't participate in GC.
+        # Explicitly remove references from the connection to its
+        # cache and to the root object, because they refer back to the
+        # connection.
+        if self._cache is not None:
+            self._cache.clear()
+        self._incrgc = None
+        self.cacheGC = None
 
     def __getitem__(self, oid, tt=type(())):
         obj = self._cache.get(oid, None)
@@ -125,11 +151,7 @@ class Connection(ExportImport.ExportImport):
         unpickler=Unpickler(file)
         unpickler.persistent_load=self._persistent_load
 
-        try:
-            object = unpickler.load()
-        except:
-            raise "Could not load oid %s, pickled data in traceback info may\
-            contain clues" % (oid)
+        object = unpickler.load()
 
         klass, args = object
 
@@ -141,7 +163,7 @@ class Connection(ExportImport.ExportImport):
             not args and not hasattr(klass,'__getinitargs__')):
             object=klass.__basicnew__()
         else:
-            object=apply(klass,args)
+            object = klass(*args)
             if klass is not ExtensionKlass:
                 object.__dict__.clear()
 
@@ -151,8 +173,6 @@ class Connection(ExportImport.ExportImport):
         object._p_serial=serial
 
         self._cache[oid] = object
-        if oid=='\0\0\0\0\0\0\0\0':
-            self._root_=object # keep a ref
         return object
 
     def _persistent_load(self,oid,
@@ -212,7 +232,7 @@ class Connection(ExportImport.ExportImport):
             # New code is in place.  Start a new cache.
             self._resetCache()
         else:
-            self._cache.invalidate(self._invalidated)
+            self._flush_invalidations()
         self._opened=time()
 
         return self
@@ -224,6 +244,7 @@ class Connection(ExportImport.ExportImport):
         self._code_timestamp = global_code_timestamp
         self._invalidated.clear()
         orig_cache = self._cache
+        orig_cache.clear()
         self._cache = cache = PickleCache(self, orig_cache.cache_size)
         self._incrgc = self.cacheGC = cache.incrgc
 
@@ -233,7 +254,7 @@ class Connection(ExportImport.ExportImport):
         This just deactivates the thing.
         """
         if object is self:
-            self._cache.invalidate(self._invalidated)
+            self._flush_invalidations()
         else:
             assert object._p_oid is not None
             self._cache.invalidate(object._p_oid)
@@ -253,8 +274,8 @@ class Connection(ExportImport.ExportImport):
         self.__onCloseCallbacks.append(f)
 
     def close(self):
-        self._incrgc() # This is a good time to do some GC
-        db=self._db
+        if self._incrgc is not None:
+            self._incrgc() # This is a good time to do some GC
 
         # Call the close callbacks.
         if self.__onCloseCallbacks is not None:
@@ -265,10 +286,10 @@ class Connection(ExportImport.ExportImport):
                     LOG('ZODB',ERROR, 'Close callback failed for %s' % f,
                         error=sys.exc_info())
             self.__onCloseCallbacks = None
-        self._db=self._storage=self._tmp=self.new_oid=self._opened=None
-        self._debug_info=()
+        self._storage = self._tmp = self.new_oid = self._opened = None
+        self._debug_info = ()
         # Return the connection to the pool.
-        db._closeConnection(self)
+        self._db._closeConnection(self)
 
     __onCommitActions = None
 
@@ -276,17 +297,21 @@ class Connection(ExportImport.ExportImport):
         if self.__onCommitActions is None:
             self.__onCommitActions = []
         self.__onCommitActions.append((method_name, args, kw))
-        get_transaction().register(self)
+        self.getTransaction().register(self)
 
     def commit(self, object, transaction):
         if object is self:
             # We registered ourself.  Execute a commit action, if any.
             if self.__onCommitActions is not None:
                 method_name, args, kw = self.__onCommitActions.pop(0)
-                apply(getattr(self, method_name), (transaction,) + args, kw)
+                getattr(self, method_name)(transaction, *args, **kw)
             return
-        
+
         oid = object._p_oid
+        if self._conflicts.has_key(oid):
+            self.getTransaction().register(object)
+            raise ReadConflictError(object=object)
+
         invalid = self._invalid
         if oid is None or object._p_jar is not self:
             # new object
@@ -296,9 +321,11 @@ class Connection(ExportImport.ExportImport):
             self._creating.append(oid)
 
         elif object._p_changed:
-            if invalid(oid) and not hasattr(object, '_p_resolveConflict'):
-                raise ConflictError(object=object)
-            self._invalidating.append(oid)
+            if invalid(oid):
+                resolve = getattr(object, "_p_resolveConflict", None)
+                if resolve is None:
+                    raise ConflictError(object=object)
+            self._modified.append(oid)
 
         else:
             # Nothing to do
@@ -360,7 +387,7 @@ class Connection(ExportImport.ExportImport):
                 #XXX We should never get here
                 if invalid(oid) and not hasattr(object, '_p_resolveConflict'):
                     raise ConflictError(object=object)
-                self._invalidating.append(oid)
+                self._modified.append(oid)
 
             klass = object.__class__
 
@@ -424,9 +451,9 @@ class Connection(ExportImport.ExportImport):
         oids=src._index.keys()
 
         # Copy invalidating and creating info from temporary storage:
-        invalidating=self._invalidating
-        invalidating[len(invalidating):]=oids
-        creating=self._creating
+        modified = self._modified
+        modified[len(modified):] = oids
+        creating = self._creating
         creating[len(creating):]=src._creating
 
         for oid in oids:
@@ -467,15 +494,31 @@ class Connection(ExportImport.ExportImport):
 
     def getVersion(self): return self._version
 
-    def invalidate(self, oid):
-        """Invalidate a particular oid
+    def isReadOnly(self):
+        return self._storage.isReadOnly()
+
+    def invalidate(self, oids):
+        """Invalidate a set of oids.
 
         This marks the oid as invalid, but doesn't actually invalidate
         it.  The object data will be actually invalidated at certain
         transaction boundaries.
         """
-        assert oid is not None
-        self._invalidated[oid] = 1
+        self._inv_lock.acquire()
+        try:
+            self._invalidated.update(oids)
+        finally:
+            self._inv_lock.release()
+
+    def _flush_invalidations(self):
+        self._inv_lock.acquire()
+        try:
+            self._cache.invalidate(self._invalidated)
+            self._invalidated.clear()
+        finally:
+            self._inv_lock.release()
+        # Now is a good time to collect some garbage
+        self._cache.incrgc()
 
     def modifiedInVersion(self, oid):
         try: return self._db.modifiedInVersion(oid)
@@ -491,74 +534,99 @@ class Connection(ExportImport.ExportImport):
         assert object._p_jar is self
         # XXX Figure out why this assert causes test failures
         # assert object._p_oid is not None
-        get_transaction().register(object)
+        self.getTransaction().register(object)
 
     def root(self):
         return self['\0\0\0\0\0\0\0\0']
 
-    def setstate(self, object):
-        oid = object._p_oid
+    def setstate(self, obj):
+        oid = obj._p_oid
 
         if self._storage is None:
             msg = ("Shouldn't load state for %s "
-                   "when the connection is closed" % `oid`)
+                   "when the connection is closed" % oid_repr(oid))
             LOG('ZODB', ERROR, msg)
             raise RuntimeError(msg)
 
         try:
+            # Avoid reading data from a transaction that committed
+            # after the current transaction started, as that might
+            # lead to mixing of cached data from earlier transactions
+            # and new inconsistent data.
+            #
+            # Wait for check until after data is loaded from storage
+            # to avoid time-of-check to time-of-use race.
             p, serial = self._storage.load(oid, self._version)
             self._load_count = self._load_count + 1
-
-            # XXX this is quite conservative!
-            # We need, however, to avoid reading data from a transaction
-            # that committed after the current "session" started, as
-            # that might lead to mixing of cached data from earlier
-            # transactions and new inconsistent data.
-            #
-            # Note that we (carefully) wait until after we call the
-            # storage to make sure that we don't miss an invaildation
-            # notifications between the time we check and the time we
-            # read.
-            if self._invalid(oid):
-                if not hasattr(object.__class__, '_p_independent'):
-                    get_transaction().register(self)
-                    raise ReadConflictError(object=object)
-                invalid = 1
-            else:
-                invalid = 0
-
-            file = StringIO(p)
-            unpickler = Unpickler(file)
-            unpickler.persistent_load = self._persistent_load
-            unpickler.load()
-            state = unpickler.load()
-
-            if hasattr(object, '__setstate__'):
-                object.__setstate__(state)
-            else:
-                d = object.__dict__
-                for k, v in state.items():
-                    d[k] = v
-
-            object._p_serial = serial
-
+            invalid = self._is_invalidated(obj)
+            self._set_ghost_state(obj, p)
+            obj._p_serial = serial
             if invalid:
-                if object._p_independent():
-                    try:
-                        del self._invalidated[oid]
-                    except KeyError:
-                        pass
-                else:
-                    get_transaction().register(self)
-                    raise ConflictError(object=object)
-
+                self._handle_independent(obj)
         except ConflictError:
             raise
         except:
-            LOG('ZODB',ERROR, "Couldn't load state for %s" % `oid`,
+            LOG('ZODB', ERROR,
+                "Couldn't load state for %s" % oid_repr(oid),
                 error=sys.exc_info())
             raise
 
+    def _is_invalidated(self, obj):
+        # Helper method for setstate() covers three cases:
+        # returns false if obj is valid
+        # returns true if obj was invalidation, but is independent
+        # otherwise, raises ConflictError for invalidated objects
+        self._inv_lock.acquire()
+        try:
+            if self._invalidated.has_key(obj._p_oid):
+                # Defer _p_independent() call until state is loaded.
+                ind = getattr(obj, "_p_independent", None)
+                if ind is not None:
+                    # Defer _p_independent() call until state is loaded.
+                    return 1
+                else:
+                    self.getTransaction().register(obj)
+                    self._conflicts[obj._p_oid] = 1
+                    raise ReadConflictError(object=obj)
+            else:
+                return 0
+        finally:
+            self._inv_lock.release()
+
+    def _set_ghost_state(self, obj, p):
+        file = StringIO(p)
+        unpickler = Unpickler(file)
+        unpickler.persistent_load = self._persistent_load
+        unpickler.load()
+        state = unpickler.load()
+
+        setstate = getattr(obj, "__setstate__", None)
+        if setstate is None:
+            obj.update(state)
+        else:
+            setstate(state)
+
+    def _handle_independent(self, obj):
+        # Helper method for setstate() handles possibly independent objects
+        # Call _p_independent(), if it returns True, setstate() wins.
+        # Otherwise, raise a ConflictError.
+
+        if obj._p_independent():
+            self._inv_lock.acquire()
+            try:
+                try:
+                    del self._invalidated[obj._p_oid]
+                except KeyError:
+                    pass
+            finally:
+                self._inv_lock.release()
+        else:
+            self.getTransaction().register(obj)
+            raise ReadConflictError(object=obj)
+
+    def _ignore_conflict(self, oid):
+        del self._conflicts[oid]
+
     def oldstate(self, object, serial):
         oid=object._p_oid
         p = self._storage.loadSerial(oid, serial)
@@ -566,7 +634,7 @@ class Connection(ExportImport.ExportImport):
         unpickler=Unpickler(file)
         unpickler.persistent_load=self._persistent_load
         unpickler.load()
-        return  unpickler.load()
+        return unpickler.load()
 
     def setklassstate(self, object):
         try:
@@ -587,7 +655,7 @@ class Connection(ExportImport.ExportImport):
                     % getattr(object,'__name__','(?)'))
                 return
 
-            copy=apply(klass,args)
+            copy = klass(*args)
             object.__dict__.clear()
             object.__dict__.update(copy.__dict__)
 
@@ -603,14 +671,14 @@ class Connection(ExportImport.ExportImport):
         if self.__onCommitActions is not None:
             del self.__onCommitActions
         self._storage.tpc_abort(transaction)
-        self._cache.invalidate(self._invalidated)
-        self._cache.invalidate(self._invalidating)
+        self._cache.invalidate(self._modified)
+        self._flush_invalidations()
+        self._conflicts.clear()
         self._invalidate_creating()
 
     def tpc_begin(self, transaction, sub=None):
-        self._invalidating = []
+        self._modified = []
         self._creating = []
-
         if sub:
             # Sub-transaction!
             if self._tmp is None:
@@ -645,40 +713,40 @@ class Connection(ExportImport.ExportImport):
         # tpc_vote() calls already did this.  The change=1 argument
         # exists to allow commit_sub() to avoid setting the flag
         # again.
+
+        # When conflict resolution occurs, the object state held by
+        # the connection does not match what is written to the
+        # database.  Invalidate the object here to guarantee that
+        # the new state is read the next time the object is used.
+
         if not store_return:
             return
         if isinstance(store_return, StringType):
             assert oid is not None
-            serial = store_return
-            obj = self._cache.get(oid, None)
-            if obj is None:
-                return
-            if serial == ResolvedSerial:
-                obj._p_changed = None
-            else:
-                if change:
-                    obj._p_changed = 0
-                obj._p_serial = serial
+            self._handle_one_serial(oid, store_return, change)
         else:
             for oid, serial in store_return:
-                if not isinstance(serial, StringType):
-                    raise serial
-                obj = self._cache.get(oid, None)
-                if obj is None:
-                    continue
-                if serial == ResolvedSerial:
-                    obj._p_changed = None
-                else:
-                    if change:
-                        obj._p_changed = 0
-                    obj._p_serial = serial
+                self._handle_one_serial(oid, serial, change)
+
+    def _handle_one_serial(self, oid, serial, change):
+        if not isinstance(serial, StringType):
+            raise serial
+        obj = self._cache.get(oid, None)
+        if obj is None:
+            return
+        if serial == ResolvedSerial:
+            del obj._p_changed # transition from changed to ghost
+        else:
+            if change:
+                obj._p_changed = 0 # trans. from changed to uptodate
+            obj._p_serial = serial
 
     def tpc_finish(self, transaction):
         # It's important that the storage call the function we pass
-        # (self._invalidate_invalidating) while it still has it's
-        # lock.  We don't want another thread to be able to read any
-        # updated data until we've had a chance to send an
-        # invalidation message to all of the other connections!
+        # while it still has it's lock.  We don't want another thread
+        # to be able to read any updated data until we've had a chance
+        # to send an invalidation message to all of the other
+        # connections!
 
         if self._tmp is not None:
             # Commiting a subtransaction!
@@ -687,25 +755,21 @@ class Connection(ExportImport.ExportImport):
             self._storage._creating[:0]=self._creating
             del self._creating[:]
         else:
-            self._db.begin_invalidation()
-            self._storage.tpc_finish(transaction,
-                                     self._invalidate_invalidating)
+            def callback():
+                d = {}
+                for oid in self._modified:
+                    d[oid] = 1
+                self._db.invalidate(d, self)
+            self._storage.tpc_finish(transaction, callback)
 
-        self._cache.invalidate(self._invalidated)
-        self._incrgc() # This is a good time to do some GC
-
-    def _invalidate_invalidating(self):
-        for oid in self._invalidating:
-            assert oid is not None
-            self._db.invalidate(oid, self)
-        self._db.finish_invalidation()
+        self._conflicts.clear()
+        self._flush_invalidations()
 
     def sync(self):
-        get_transaction().abort()
+        self.getTransaction().abort()
         sync=getattr(self._storage, 'sync', 0)
         if sync != 0: sync()
-        self._cache.invalidate(self._invalidated)
-        self._incrgc() # This is a good time to do some GC
+        self._flush_invalidations()
 
     def getDebugInfo(self):
         return self._debug_info
@@ -732,7 +796,7 @@ class Connection(ExportImport.ExportImport):
         new._p_oid=oid
         new._p_jar=self
         new._p_changed=1
-        get_transaction().register(new)
+        self.getTransaction().register(new)
         self._cache[oid]=new
 
 class tConnection(Connection):
