@@ -13,14 +13,14 @@
 ##############################################################################
 """Transaction management
 
-$Id: Transaction.py,v 1.37.6.3 2003/01/30 23:25:13 bwarsaw Exp $"""
-__version__='$Revision: 1.37.6.3 $'[11:-2]
+$Id: Transaction.py,v 1.49.4.4 2004/06/09 18:42:08 tim_one Exp $
+"""
 
 import time, sys, struct, POSException
-from struct import pack
 from string import split, strip, join
 from zLOG import LOG, ERROR, PANIC, INFO, BLATHER, WARNING
 from POSException import ConflictError
+from utils import oid_repr
 
 # Flag indicating whether certain errors have occurred.
 hosed=0
@@ -43,16 +43,15 @@ def jar_cmp(j1, j2):
     except:
         LOG("TM", WARNING, "jar missing sortKey() method: %s" % j2)
         k2 = id(j2)
-        
+
     return cmp(k1, k2)
 
 class Transaction:
-    'Simple transaction objects for single-threaded applications.'
-    user=''
-    description=''
-    _connections=None
-    _extension=None
-    _sub=None # This is a subtrasaction flag
+    user = ''
+    description = ''
+    _connections = None
+    _extension = None
+    _sub = None # This is a subtrasaction flag
 
     # The _non_st_objects variable is either None or a list
     # of jars that do not support subtransactions. This is used to
@@ -92,15 +91,15 @@ class Transaction:
             return "Transaction thread=%s user=%s" % (self._id, `self.user`)
 
     def __del__(self):
-        if self._objects: self.abort(freeme=0)
+        if self._objects:
+            self.abort(freeme=0)
 
     def abort(self, subtransaction=0, freeme=1):
-        '''Abort the transaction.
+        """Abort the transaction.
 
-        This is called from the application.  This means that we haven\'t
+        This is called from the application.  This means that we haven't
         entered two-phase commit yet, so no tpc_ messages are sent.
-        '''
-
+        """
         if subtransaction and (self._non_st_objects is not None):
             raise POSException.TransactionError, (
                 """Attempted to abort a sub-transaction, but a participating
@@ -138,8 +137,11 @@ class Transaction:
                     if t is None:
                         t, v, tb = sys.exc_info()
                     else:
-                        self.log("Failed to abort object %s" %
-                                 repr(o._p_oid), error=sys.exc_info())
+                        if getattr(o,'_p_oid',None):
+                            msg = "Failed to abort object %s" % repr(o._p_oid)
+                        else:
+                            msg = "Failed to abort object"
+                        self.log(msg, error=sys.exc_info())
 
             # tpc_begin() was never called, so tpc_abort() should not be
             # called.
@@ -163,10 +165,10 @@ class Transaction:
                 self._init()
 
     def begin(self, info=None, subtransaction=None):
-        '''Begin a new transaction.
+        """Begin a new transaction.
 
         This aborts any transaction in progres.
-        '''
+        """
         if self._objects:
             self.abort(subtransaction, 0)
         if info:
@@ -175,8 +177,7 @@ class Transaction:
             self.description=strip(join(info[1:],'\t'))
 
     def commit(self, subtransaction=None):
-        'Finalize the transaction'
-
+        """Finalize the transaction."""
         objects = self._objects
 
         subjars = []
@@ -257,14 +258,15 @@ class Transaction:
                 # have to clean up.  First save the original exception
                 # in case the cleanup process causes another
                 # exception.
+                error = sys.exc_info()
                 try:
                     self._commit_error(objects, ncommitted, jars, subjars)
                 except:
                     LOG('ZODB', ERROR,
                         "A storage error occured during transaction "
                         "abort.  This shouldn't happen.",
-                        error=sys.exc_info())
-                raise
+                        error=error)
+                raise error[0], error[1], error[2]
         finally:
             del objects[:] # clear registered
             if not subtransaction and self._id is not None:
@@ -272,12 +274,12 @@ class Transaction:
 
     def _get_jars(self, objects, subtransaction):
         # Returns a list of jars for this transaction.
-        
+
         # Find all the jars and sort them in a globally consistent order.
         # objects is a list of persistent objects and jars.
         # If this is a subtransaction and a jar is not subtransaction aware,
         # it's object gets delayed until the parent transaction commits.
-        
+
         d = {}
         for o in objects:
             jar = getattr(o, '_p_jar', o)
@@ -299,7 +301,7 @@ class Transaction:
                     if self._non_st_objects is None:
                         self._non_st_objects = []
                     self._non_st_objects.append(o)
-                
+
         jars = d.values()
         jars.sort(jar_cmp)
 
@@ -369,6 +371,8 @@ class Transaction:
                 # The database can't guarantee consistency if call fails.
                 jar.tpc_finish(self)
         except:
+            # XXX We should consult ZConfig to decide whether we want to put
+            # the transaction manager in a hosed state or not.
             #hosed = 1
             LOG('ZODB', PANIC,
                 "A storage error occurred in the last phase of a "
@@ -389,8 +393,11 @@ class Transaction:
                     j.abort(o, self)
             except:
                 # nothing to do but log the error
-                self.log("Failed to abort object %s" % repr(o._p_oid),
-                         error=sys.exc_info())
+                if getattr(o,'_p_oid',None):
+                    msg = "Failed to abort object %s" % repr(o._p_oid)
+                else:
+                    msg = "Failed to abort object"
+                self.log(msg, error=sys.exc_info())
 
         # Abort the two-phase commit.  It's only necessary to abort the
         # commit for jars that began it, but it is harmless to abort it
@@ -405,7 +412,7 @@ class Transaction:
 
         # After the tpc_abort(), call abort_sub() on all the
         # subtrans-aware jars to *really* abort the subtransaction.
-        
+
         # Example: For Connection(), the tpc_abort() will abort the
         # subtransaction TmpStore() and abort_sub() will remove the
         # TmpStore.
