@@ -10,10 +10,14 @@
 # FOR A PARTICULAR PURPOSE
 #
 ##############################################################################
-__version__='$Revision: 1.50 $'[11:-2]
+""" Basic ZPublisher request management.
+
+$Id: BaseRequest.py,v 1.51.2.2 2003/11/17 22:34:19 tseaver Exp $
+"""
 
 from urllib import quote
 import xmlrpc
+from zExceptions import Forbidden
 
 UNSPECIFIED_ROLES=''
 
@@ -269,13 +273,13 @@ class BaseRequest:
                         request['TraversalRequestNameStack'] = path
                         continue
                     else:
-                        method = default_path[0]
-                        entry_name = method
+                        entry_name = default_path[0]
                 elif (method and hasattr(object,method)
                       and entry_name != method
                       and getattr(object, method) is not None):
                     request._hacked_path=1
                     entry_name = method
+                    method = 'index_html'
                 else:
                     if (hasattr(object, '__call__') and
                         hasattr(object.__call__,'__roles__')):
@@ -284,7 +288,6 @@ class BaseRequest:
                         i=URL.rfind('/')
                         if i > 0: response.setBase(URL[:i])
                     break
-                if not entry_name: continue
                 step = quote(entry_name)
                 _steps.append(step)
                 request['URL'] = URL = '%s/%s' % (request['URL'], step)
@@ -344,6 +347,17 @@ class BaseRequest:
                         "published." % URL
                         )
 
+                # Hack for security: in Python 2.2.2, most built-in types
+                # gained docstrings that they didn't have before. That caused
+                # certain mutable types (dicts, lists) to become publishable
+                # when they shouldn't be. The following check makes sure that
+                # the right thing happens in both 2.2.2+ and earlier versions.
+                
+                if not typeCheck(subobject):
+                    return response.debugError(
+                        "The object at %s is not publishable." % URL
+                        )
+
                 r = getattr(subobject, '__roles__', UNSPECIFIED_ROLES)
                 if r is not UNSPECIFIED_ROLES:
                     roles = r
@@ -486,7 +500,31 @@ def old_validation(groups, request, auth,
 
     if keys is None:
         # Not a named group, so don't go further
-        raise 'Forbidden', (
+        raise Forbidden, (
             """<strong>You are not authorized to access this resource""")
 
     return None
+
+
+
+# This mapping contains the built-in types that gained docstrings
+# between Python 2.1 and 2.2.2. By specifically checking for these
+# types during publishing, we ensure the same publishing rules in
+# both versions. The downside is that this needs to be extended as
+# new built-in types are added and future Python versions are
+# supported. That happens rarely enough that hopefully we'll be on
+# Zope 3 by then :)
+
+import types
+
+itypes = {}
+for name in ('NoneType', 'IntType', 'LongType', 'FloatType', 'StringType',
+             'BufferType', 'TupleType', 'ListType', 'DictType', 'XRangeType',
+             'SliceType', 'EllipsisType', 'UnicodeType', 'CodeType',
+             'TracebackType', 'FrameType', 'DictProxyType'):
+    if hasattr(types, name):
+        itypes[getattr(types, name)] = 0
+
+def typeCheck(obj, deny=itypes):
+    # Return true if its ok to publish the type, false otherwise.
+    return deny.get(type(obj), 1)
