@@ -12,16 +12,17 @@
 ##############################################################################
 '''CGI Response Output formatter
 
-$Id: HTTPResponse.py,v 1.69.4.3 2002/12/22 17:39:02 efge Exp $'''
-__version__ = '$Revision: 1.69.4.3 $'[11:-2]
+$Id: HTTPResponse.py,v 1.75.2.6 2004/04/26 10:11:23 shh Exp $'''
+__version__ = '$Revision: 1.75.2.6 $'[11:-2]
 
 import types, os, sys, re
 import zlib, struct
 from string import translate, maketrans
-from types import StringType, InstanceType, LongType, UnicodeType
 from BaseResponse import BaseResponse
-from zExceptions import Unauthorized
+from zExceptions import Unauthorized, Redirect
 from zExceptions.ExceptionFormatter import format_exception
+from ZPublisher import BadRequest, InternalError, NotFound
+from cgi import escape
 
 nl2sp = maketrans('\n',' ')
 
@@ -190,7 +191,8 @@ class HTTPResponse(BaseResponse):
     def _requestShutdown(self, exitCode=0):
         """Request that the server shut down with exitCode after fulfilling
            the current request."""
-        sys.ZServerExitCode = exitCode
+        import ZServer
+        ZServer.exit_code = exitCode
         self._shutdown_flag = 1
 
     def _shutdownRequested(self):
@@ -211,13 +213,20 @@ class HTTPResponse(BaseResponse):
             # It has already been determined.
             return
 
-        if type(status) is types.StringType:
+        if (isinstance(status, types.ClassType)
+         and issubclass(status, Exception)):
+            status = status.__name__
+
+        if isinstance(status, str):
             status = status.lower()
+
         if status_codes.has_key(status):
             status = status_codes[status]
         else:
             status = 500
+
         self.status = status
+
         if reason is None:
             if status_reasons.has_key(status):
                 reason = status_reasons[status]
@@ -279,16 +288,16 @@ class HTTPResponse(BaseResponse):
         if not body:
             return self
 
-        if type(body) is types.TupleType and len(body) == 2:
+        if isinstance(body, tuple) and len(body) == 2:
             title,body = body
 
-        if type(body) is not types.StringType:
+        if not isinstance(body, str):
             if hasattr(body,'asHTML'):
                 body = body.asHTML()
 
-        if type(body) is UnicodeType:
+        if isinstance(body, unicode):
             body = self._encode_unicode(body)
-        elif type(body) is StringType:
+        elif isinstance(body, str):
             pass
         else:
             try:
@@ -453,7 +462,8 @@ class HTTPResponse(BaseResponse):
                     ibase = base_re_search(body)
                     if ibase is None:
                         self.body = ('%s\n<base href="%s" />\n%s' %
-                                   (body[:index], self.base, body[index:]))
+                                   (body[:index], escape(self.base, 1),
+                                    body[index:]))
                         self.setHeader('content-length', len(self.body))
 
     def appendCookie(self, name, value):
@@ -490,10 +500,10 @@ class HTTPResponse(BaseResponse):
         '''
         name = str(name)
 
-        dict = {'max_age':0, 'expires':'Wed, 31-Dec-97 23:59:59 GMT'}
-        for k, v in kw.items():
-            dict[k] = v
-        apply(HTTPResponse.setCookie, (self, name, 'deleted'), dict)
+        d = kw.copy()
+        d['max_age'] = 0
+        d['expires'] = 'Wed, 31-Dec-97 23:59:59 GMT'
+        apply(HTTPResponse.setCookie, (self, name, 'deleted'), d)
 
     def setCookie(self,name,value,**kw):
         '''\
@@ -544,15 +554,9 @@ class HTTPResponse(BaseResponse):
             return 1
         return 0
 
-    def quoteHTML(self,text,
-                  subs={'&':'&amp;', "<":'&lt;', ">":'&gt;', '\"':'&quot;'}
-                  ):
-        for ent in '&<>\"':
-            if text.find( ent) >= 0:
-                text = subs[ent].join(text.split(ent))
-
-        return text
-
+    # deprecated
+    def quoteHTML(self, text):
+        return escape(text, 1)
 
     def _traceback(self, t, v, tb, as_html=1):
         tb = format_exception(t, v, tb, as_html=as_html)
@@ -610,7 +614,7 @@ class HTTPResponse(BaseResponse):
   </UL>
 
   <P>For more detailed information about the error, please
-  refer to the HTML source for this page.
+  refer to error log.
   </P>
 
   <P>If the error persists please contact the site maintainer.
@@ -621,17 +625,17 @@ class HTTPResponse(BaseResponse):
 
     def notFoundError(self,entry='Unknown'):
         self.setStatus(404)
-        raise 'NotFound',self._error_html(
+        raise NotFound, self._error_html(
             "Resource not found",
             "Sorry, the requested resource does not exist." +
             "<p>Check the URL and try again.</p>" +
-            "<p><b>Resource:</b> %s</p>" % self.quoteHTML(entry))
+            "<p><b>Resource:</b> %s</p>" % escape(entry))
 
     forbiddenError = notFoundError  # If a resource is forbidden,
                                     # why reveal that it exists?
 
     def debugError(self,entry):
-        raise 'NotFound',self._error_html(
+        raise NotFound, self._error_html(
             "Debugging Notice",
             "Zope has encountered a problem publishing your object.<p>"
             "\n%s" % entry)
@@ -639,11 +643,11 @@ class HTTPResponse(BaseResponse):
     def badRequestError(self,name):
         self.setStatus(400)
         if re.match('^[A-Z_0-9]+$',name):
-            raise 'InternalError', self._error_html(
+            raise InternalError, self._error_html(
                 "Internal Error",
                 "Sorry, an internal error occurred in this resource.")
 
-        raise 'BadRequest',self._error_html(
+        raise BadRequest, self._error_html(
             "Invalid request",
             "The parameter, <em>%s</em>, " % name +
             "was omitted from the request.<p>" +
@@ -687,15 +691,24 @@ class HTTPResponse(BaseResponse):
             # Try to capture exception info for bci calls
             et = translate(str(t), nl2sp)
             self.setHeader('bobo-exception-type', et)
-            ev = translate(str(v), nl2sp)
-            if ev.find( '<html>') >= 0:
-                ev = 'bobo exception'
-            self.setHeader('bobo-exception-value', ev[:255])
+
+            # As of Zope 2.6.2 / 2.7, we no longer try to pass along a
+            # meaningful exception value. Now that there are good logging
+            # facilities on the server side (and given that xml-rpc has
+            # largely removed the need for this code at all), we just
+            # refer the caller to the server error log.
+            ev = 'See the server error log for details'
+            self.setHeader('bobo-exception-value', ev)
+
             # Get the tb tail, which is the interesting part:
             while tb.tb_next is not None:
                 tb = tb.tb_next
             el = str(tb.tb_lineno)
             ef = str(tb.tb_frame.f_code.co_filename)
+
+            # Do not give out filesystem information.
+            ef = ef.split(os.sep)[-1]
+
             self.setHeader('bobo-exception-file', ef)
             self.setHeader('bobo-exception-line', el)
 
@@ -707,16 +720,23 @@ class HTTPResponse(BaseResponse):
         del stb
         self.setStatus(t)
         if self.status >= 300 and self.status < 400:
-            if type(v) == types.StringType and absuri_match(v) is not None:
+            if isinstance(v, str) and absuri_match(v) is not None:
                 if self.status == 300:
                     self.setStatus(302)
                 self.setHeader('location', v)
                 tb = None # just one path covered
                 return self
+            elif isinstance(v, Redirect): # death to string exceptions!
+                if self.status == 300:
+                    self.setStatus(302)
+                self.setHeader('location', v.args[0])
+                self.setBody('')
+                tb = None
+                return self
             else:
                 try:
                     l, b = v
-                    if (type(l) == types.StringType
+                    if (isinstance(l, str)
                         and absuri_match(l) is not None):
                         if self.status == 300:
                             self.setStatus(302)
